@@ -3,35 +3,37 @@ from textwrap import dedent
 import _pytest._code
 import pytest
 import sys
-from _pytest import python as funcargs
 from _pytest.pytester import get_public_names
-from _pytest.python import FixtureLookupError
-
+from _pytest.fixtures import FixtureLookupError
+from _pytest import fixtures
 
 def test_getfuncargnames():
     def f(): pass
-    assert not funcargs.getfuncargnames(f)
+    assert not fixtures.getfuncargnames(f)
     def g(arg): pass
-    assert funcargs.getfuncargnames(g) == ('arg',)
+    assert fixtures.getfuncargnames(g) == ('arg',)
     def h(arg1, arg2="hello"): pass
-    assert funcargs.getfuncargnames(h) == ('arg1',)
+    assert fixtures.getfuncargnames(h) == ('arg1',)
     def h(arg1, arg2, arg3="hello"): pass
-    assert funcargs.getfuncargnames(h) == ('arg1', 'arg2')
+    assert fixtures.getfuncargnames(h) == ('arg1', 'arg2')
     class A:
         def f(self, arg1, arg2="hello"):
             pass
-    assert funcargs.getfuncargnames(A().f) == ('arg1',)
+    assert fixtures.getfuncargnames(A().f) == ('arg1',)
     if sys.version_info < (3,0):
-        assert funcargs.getfuncargnames(A.f) == ('arg1',)
+        assert fixtures.getfuncargnames(A.f) == ('arg1',)
 
 class TestFillFixtures:
     def test_fillfuncargs_exposed(self):
         # used by oejskit, kept for compatibility
-        assert pytest._fillfuncargs == funcargs.fillfixtures
+        assert pytest._fillfuncargs == fixtures.fillfixtures
 
     def test_funcarg_lookupfails(self, testdir):
         testdir.makepyfile("""
-            def pytest_funcarg__xyzsomething(request):
+            import pytest
+
+            @pytest.fixture
+            def xyzsomething(request):
                 return 42
 
             def test_func(some):
@@ -47,14 +49,18 @@ class TestFillFixtures:
 
     def test_funcarg_basic(self, testdir):
         item = testdir.getitem("""
-            def pytest_funcarg__some(request):
+            import pytest
+
+            @pytest.fixture
+            def some(request):
                 return request.function.__name__
-            def pytest_funcarg__other(request):
+            @pytest.fixture
+            def other(request):
                 return 42
             def test_func(some, other):
                 pass
         """)
-        funcargs.fillfixtures(item)
+        fixtures.fillfixtures(item)
         del item.funcargs["request"]
         assert len(get_public_names(item.funcargs)) == 2
         assert item.funcargs['some'] == "test_func"
@@ -62,7 +68,10 @@ class TestFillFixtures:
 
     def test_funcarg_lookup_modulelevel(self, testdir):
         testdir.makepyfile("""
-            def pytest_funcarg__something(request):
+            import pytest
+
+            @pytest.fixture
+            def something(request):
                 return request.function.__name__
 
             class TestClass:
@@ -76,9 +85,13 @@ class TestFillFixtures:
 
     def test_funcarg_lookup_classlevel(self, testdir):
         p = testdir.makepyfile("""
+            import pytest
             class TestClass:
-                def pytest_funcarg__something(self, request):
+
+                @pytest.fixture
+                def something(self, request):
                     return request.instance
+
                 def test_method(self, something):
                     assert something is self
         """)
@@ -92,13 +105,15 @@ class TestFillFixtures:
         sub2 = testdir.mkpydir("sub2")
         sub1.join("conftest.py").write(_pytest._code.Source("""
             import pytest
-            def pytest_funcarg__arg1(request):
-                pytest.raises(Exception, "request.getfuncargvalue('arg2')")
+            @pytest.fixture
+            def arg1(request):
+                pytest.raises(Exception, "request.getfixturevalue('arg2')")
         """))
         sub2.join("conftest.py").write(_pytest._code.Source("""
             import pytest
-            def pytest_funcarg__arg2(request):
-                pytest.raises(Exception, "request.getfuncargvalue('arg1')")
+            @pytest.fixture
+            def arg2(request):
+                pytest.raises(Exception, "request.getfixturevalue('arg1')")
         """))
 
         sub1.join("test_in_sub1.py").write("def test_1(arg1): pass")
@@ -336,6 +351,38 @@ class TestFillFixtures:
         result = testdir.runpytest(testfile)
         result.stdout.fnmatch_lines(["*3 passed*"])
 
+    def test_override_autouse_fixture_with_parametrized_fixture_conftest_conftest(self, testdir):
+        """Test override of the autouse fixture with parametrized one on the conftest level.
+        This test covers the issue explained in issue 1601
+        """
+        testdir.makeconftest("""
+            import pytest
+
+            @pytest.fixture(autouse=True)
+            def spam():
+                return 'spam'
+        """)
+        subdir = testdir.mkpydir('subdir')
+        subdir.join("conftest.py").write(_pytest._code.Source("""
+            import pytest
+
+            @pytest.fixture(params=[1, 2, 3])
+            def spam(request):
+                return request.param
+        """))
+        testfile = subdir.join("test_spam.py")
+        testfile.write(_pytest._code.Source("""
+            params = {'spam': 1}
+
+            def test_spam(spam):
+                assert spam == params['spam']
+                params['spam'] += 1
+        """))
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines(["*3 passed*"])
+        result = testdir.runpytest(testfile)
+        result.stdout.fnmatch_lines(["*3 passed*"])
+
     def test_autouse_fixture_plugin(self, testdir):
         # A fixture from a plugin has no baseid set, which screwed up
         # the autouse fixture handling.
@@ -357,20 +404,37 @@ class TestFillFixtures:
         assert result.ret == 0
 
     def test_funcarg_lookup_error(self, testdir):
+        testdir.makeconftest("""
+            import pytest
+
+            @pytest.fixture
+            def a_fixture(): pass
+
+            @pytest.fixture
+            def b_fixture(): pass
+
+            @pytest.fixture
+            def c_fixture(): pass
+
+            @pytest.fixture
+            def d_fixture(): pass
+        """)
         testdir.makepyfile("""
             def test_lookup_error(unknown):
                 pass
         """)
         result = testdir.runpytest()
         result.stdout.fnmatch_lines([
-            "*ERROR*test_lookup_error*",
-            "*def test_lookup_error(unknown):*",
-            "*fixture*unknown*not found*",
-            "*available fixtures*",
+            "*ERROR at setup of test_lookup_error*",
+            "  def test_lookup_error(unknown):*",
+            "E       fixture 'unknown' not found",
+            ">       available fixtures:*a_fixture,*b_fixture,*c_fixture,*d_fixture*monkeypatch,*",  # sorted
+            ">       use 'py*test --fixtures *' for help on them.",
             "*1 error*",
         ])
         assert "INTERNAL" not in result.stdout.str()
 
+
     def test_fixture_excinfo_leak(self, testdir):
         # on python2 sys.excinfo would leak into fixture executions
         testdir.makepyfile("""
@@ -397,10 +461,13 @@ class TestFillFixtures:
 class TestRequestBasic:
     def test_request_attributes(self, testdir):
         item = testdir.getitem("""
-            def pytest_funcarg__something(request): pass
+            import pytest
+
+            @pytest.fixture
+            def something(request): pass
             def test_func(something): pass
         """)
-        req = funcargs.FixtureRequest(item)
+        req = fixtures.FixtureRequest(item)
         assert req.function == item.obj
         assert req.keywords == item.keywords
         assert hasattr(req.module, 'test_func')
@@ -411,8 +478,11 @@ class TestRequestBasic:
 
     def test_request_attributes_method(self, testdir):
         item, = testdir.getitems("""
+            import pytest
             class TestB:
-                def pytest_funcarg__something(self, request):
+
+                @pytest.fixture
+                def something(self, request):
                     return 1
                 def test_func(self, something):
                     pass
@@ -421,9 +491,11 @@ class TestRequestBasic:
         assert req.cls.__name__ == "TestB"
         assert req.instance.__class__ == req.cls
 
-    def XXXtest_request_contains_funcarg_arg2fixturedefs(self, testdir):
+    def test_request_contains_funcarg_arg2fixturedefs(self, testdir):
         modcol = testdir.getmodulecol("""
-            def pytest_funcarg__something(request):
+            import pytest
+            @pytest.fixture
+            def something(request):
                 pass
             class TestClass:
                 def test_method(self, something):
@@ -431,41 +503,53 @@ class TestRequestBasic:
         """)
         item1, = testdir.genitems([modcol])
         assert item1.name == "test_method"
-        arg2fixturedefs = funcargs.FixtureRequest(item1)._arg2fixturedefs
+        arg2fixturedefs = fixtures.FixtureRequest(item1)._arg2fixturedefs
         assert len(arg2fixturedefs) == 1
-        assert arg2fixturedefs[0].__name__ == "pytest_funcarg__something"
+        assert arg2fixturedefs['something'][0].argname == "something"
 
-    def test_getfuncargvalue_recursive(self, testdir):
+    def test_getfixturevalue_recursive(self, testdir):
         testdir.makeconftest("""
-            def pytest_funcarg__something(request):
+            import pytest
+
+            @pytest.fixture
+            def something(request):
                 return 1
         """)
         testdir.makepyfile("""
-            def pytest_funcarg__something(request):
-                return request.getfuncargvalue("something") + 1
+            import pytest
+
+            @pytest.fixture
+            def something(request):
+                return request.getfixturevalue("something") + 1
             def test_func(something):
                 assert something == 2
         """)
         reprec = testdir.inline_run()
         reprec.assertoutcome(passed=1)
 
-    def test_getfuncargvalue(self, testdir):
+    @pytest.mark.parametrize(
+        'getfixmethod', ('getfixturevalue', 'getfuncargvalue'))
+    def test_getfixturevalue(self, testdir, getfixmethod):
         item = testdir.getitem("""
+            import pytest
             l = [2]
-            def pytest_funcarg__something(request): return 1
-            def pytest_funcarg__other(request):
+            @pytest.fixture
+            def something(request): return 1
+            @pytest.fixture
+            def other(request):
                 return l.pop()
             def test_func(something): pass
         """)
         req = item._request
-        pytest.raises(FixtureLookupError, req.getfuncargvalue, "notexists")
-        val = req.getfuncargvalue("something")
+        fixture_fetcher = getattr(req, getfixmethod)
+        pytest.raises(FixtureLookupError, fixture_fetcher, "notexists")
+        val = fixture_fetcher("something")
         assert val == 1
-        val = req.getfuncargvalue("something")
+        val = fixture_fetcher("something")
         assert val == 1
-        val2 = req.getfuncargvalue("other")
+        val2 = fixture_fetcher("other")
         assert val2 == 2
-        val2 = req.getfuncargvalue("other")  # see about caching
+        val2 = fixture_fetcher("other")  # see about caching
         assert val2 == 2
         pytest._fillfuncargs(item)
         assert item.funcargs["something"] == 1
@@ -475,8 +559,10 @@ class TestRequestBasic:
 
     def test_request_addfinalizer(self, testdir):
         item = testdir.getitem("""
+            import pytest
             teardownlist = []
-            def pytest_funcarg__something(request):
+            @pytest.fixture
+            def something(request):
                 request.addfinalizer(lambda: teardownlist.append(1))
             def test_func(something): pass
         """)
@@ -490,6 +576,21 @@ class TestRequestBasic:
         print(ss.stack)
         assert teardownlist == [1]
 
+    def test_mark_as_fixture_with_prefix_and_decorator_fails(self, testdir):
+        testdir.makeconftest("""
+            import pytest
+
+            @pytest.fixture
+            def pytest_funcarg__marked_with_prefix_and_decorator():
+                pass
+        """)
+        result = testdir.runpytest_subprocess()
+        assert result.ret != 0
+        result.stdout.fnmatch_lines([
+            "*AssertionError: fixtures cannot have*@pytest.fixture*",
+            "*pytest_funcarg__marked_with_prefix_and_decorator*"
+        ])
+
     def test_request_addfinalizer_failing_setup(self, testdir):
         testdir.makepyfile("""
             import pytest
@@ -525,8 +626,10 @@ class TestRequestBasic:
 
     def test_request_addfinalizer_partial_setup_failure(self, testdir):
         p = testdir.makepyfile("""
+            import pytest
             l = []
-            def pytest_funcarg__something(request):
+            @pytest.fixture
+            def something(request):
                 request.addfinalizer(lambda: l.append(None))
             def test_func(something, missingarg):
                 pass
@@ -541,7 +644,7 @@ class TestRequestBasic:
     def test_request_getmodulepath(self, testdir):
         modcol = testdir.getmodulecol("def test_somefunc(): pass")
         item, = testdir.genitems([modcol])
-        req = funcargs.FixtureRequest(item)
+        req = fixtures.FixtureRequest(item)
         assert req.fspath == modcol.fspath
 
     def test_request_fixturenames(self, testdir):
@@ -567,9 +670,11 @@ class TestRequestBasic:
 
     def test_funcargnames_compatattr(self, testdir):
         testdir.makepyfile("""
+            import pytest
             def pytest_generate_tests(metafunc):
                 assert metafunc.funcargnames == metafunc.fixturenames
-            def pytest_funcarg__fn(request):
+            @pytest.fixture
+            def fn(request):
                 assert request._pyfuncitem.funcargnames == \
                        request._pyfuncitem.fixturenames
                 return request.funcargnames, request.fixturenames
@@ -614,7 +719,9 @@ class TestRequestBasic:
         # this tests that normalization of nodeids takes place
         b = testdir.mkdir("tests").mkdir("unit")
         b.join("conftest.py").write(_pytest._code.Source("""
-            def pytest_funcarg__arg1():
+            import pytest
+            @pytest.fixture
+            def arg1():
                 pass
         """))
         p = b.join("test_module.py")
@@ -662,7 +769,10 @@ class TestRequestBasic:
 class TestRequestMarking:
     def test_applymarker(self, testdir):
         item1,item2 = testdir.getitems("""
-            def pytest_funcarg__something(request):
+            import pytest
+
+            @pytest.fixture
+            def something(request):
                 pass
             class TestClass:
                 def test_func1(self, something):
@@ -670,7 +780,7 @@ class TestRequestMarking:
                 def test_func2(self, something):
                     pass
         """)
-        req1 = funcargs.FixtureRequest(item1)
+        req1 = fixtures.FixtureRequest(item1)
         assert 'xfail' not in item1.keywords
         req1.applymarker(pytest.mark.xfail)
         assert 'xfail' in item1.keywords
@@ -721,7 +831,10 @@ class TestRequestCachedSetup:
         reprec = testdir.inline_runsource("""
             mysetup = ["hello",].pop
 
-            def pytest_funcarg__something(request):
+            import pytest
+
+            @pytest.fixture
+            def something(request):
                 return request.cached_setup(mysetup, scope="module")
 
             def test_func1(something):
@@ -736,7 +849,9 @@ class TestRequestCachedSetup:
         reprec = testdir.inline_runsource("""
             mysetup = ["hello", "hello2", "hello3"].pop
 
-            def pytest_funcarg__something(request):
+            import pytest
+            @pytest.fixture
+            def something(request):
                 return request.cached_setup(mysetup, scope="class")
             def test_func1(something):
                 assert something == "hello3"
@@ -752,7 +867,7 @@ class TestRequestCachedSetup:
 
     def test_request_cachedsetup_extrakey(self, testdir):
         item1 = testdir.getitem("def test_func(): pass")
-        req1 = funcargs.FixtureRequest(item1)
+        req1 = fixtures.FixtureRequest(item1)
         l = ["hello", "world"]
         def setup():
             return l.pop()
@@ -767,7 +882,7 @@ class TestRequestCachedSetup:
 
     def test_request_cachedsetup_cache_deletion(self, testdir):
         item1 = testdir.getitem("def test_func(): pass")
-        req1 = funcargs.FixtureRequest(item1)
+        req1 = fixtures.FixtureRequest(item1)
         l = []
         def setup():
             l.append("setup")
@@ -786,9 +901,13 @@ class TestRequestCachedSetup:
 
     def test_request_cached_setup_two_args(self, testdir):
         testdir.makepyfile("""
-            def pytest_funcarg__arg1(request):
+            import pytest
+
+            @pytest.fixture
+            def arg1(request):
                 return request.cached_setup(lambda: 42)
-            def pytest_funcarg__arg2(request):
+            @pytest.fixture
+            def arg2(request):
                 return request.cached_setup(lambda: 17)
             def test_two_different_setups(arg1, arg2):
                 assert arg1 != arg2
@@ -798,12 +917,16 @@ class TestRequestCachedSetup:
             "*1 passed*"
         ])
 
-    def test_request_cached_setup_getfuncargvalue(self, testdir):
+    def test_request_cached_setup_getfixturevalue(self, testdir):
         testdir.makepyfile("""
-            def pytest_funcarg__arg1(request):
-                arg1 = request.getfuncargvalue("arg2")
+            import pytest
+
+            @pytest.fixture
+            def arg1(request):
+                arg1 = request.getfixturevalue("arg2")
                 return request.cached_setup(lambda: arg1 + 1)
-            def pytest_funcarg__arg2(request):
+            @pytest.fixture
+            def arg2(request):
                 return request.cached_setup(lambda: 10)
             def test_two_funcarg(arg1):
                 assert arg1 == 11
@@ -815,8 +938,10 @@ class TestRequestCachedSetup:
 
     def test_request_cached_setup_functional(self, testdir):
         testdir.makepyfile(test_0="""
+            import pytest
             l = []
-            def pytest_funcarg__something(request):
+            @pytest.fixture
+            def something(request):
                 val = request.cached_setup(fsetup, fteardown)
                 return val
             def fsetup(mycache=[1]):
@@ -842,7 +967,10 @@ class TestRequestCachedSetup:
 
     def test_issue117_sessionscopeteardown(self, testdir):
         testdir.makepyfile("""
-            def pytest_funcarg__app(request):
+            import pytest
+
+            @pytest.fixture
+            def app(request):
                 app = request.cached_setup(
                     scope='session',
                     setup=lambda: 0,
@@ -1103,16 +1231,23 @@ class TestFixtureUsages:
 
 
 class TestFixtureManagerParseFactories:
-    def pytest_funcarg__testdir(self, request):
-        testdir = request.getfuncargvalue("testdir")
+
+    @pytest.fixture
+    def testdir(self, request):
+        testdir = request.getfixturevalue("testdir")
         testdir.makeconftest("""
-            def pytest_funcarg__hello(request):
+            import pytest
+
+            @pytest.fixture
+            def hello(request):
                 return "conftest"
 
-            def pytest_funcarg__fm(request):
+            @pytest.fixture
+            def fm(request):
                 return request._fixturemanager
 
-            def pytest_funcarg__item(request):
+            @pytest.fixture
+            def item(request):
                 return request._pyfuncitem
         """)
         return testdir
@@ -1138,17 +1273,21 @@ class TestFixtureManagerParseFactories:
                     faclist = fm.getfixturedefs(name, item.nodeid)
                     assert len(faclist) == 1
                     fac = faclist[0]
-                    assert fac.func.__name__ == "pytest_funcarg__" + name
+                    assert fac.func.__name__ == name
         """)
         reprec = testdir.inline_run("-s")
         reprec.assertoutcome(passed=1)
 
     def test_parsefactories_conftest_and_module_and_class(self, testdir):
         testdir.makepyfile("""
-            def pytest_funcarg__hello(request):
+            import pytest
+
+            @pytest.fixture
+            def hello(request):
                 return "module"
             class TestClass:
-                def pytest_funcarg__hello(self, request):
+                @pytest.fixture
+                def hello(self, request):
                     return "class"
                 def test_hello(self, item, fm):
                     faclist = fm.getfixturedefs("hello", item.nodeid)
@@ -1196,7 +1335,9 @@ class TestFixtureManagerParseFactories:
 
 
 class TestAutouseDiscovery:
-    def pytest_funcarg__testdir(self, testdir):
+
+    @pytest.fixture
+    def testdir(self, testdir):
         testdir.makeconftest("""
             import pytest
             @pytest.fixture(autouse=True)
@@ -1210,10 +1351,12 @@ class TestAutouseDiscovery:
             def perfunction2(arg1):
                 pass
 
-            def pytest_funcarg__fm(request):
+            @pytest.fixture
+            def fm(request):
                 return request._fixturemanager
 
-            def pytest_funcarg__item(request):
+            @pytest.fixture
+            def item(request):
                 return request._pyfuncitem
         """)
         return testdir
@@ -1492,7 +1635,8 @@ class TestAutouseManagement:
                 def test_2(self):
                     pass
         """)
-        reprec = testdir.inline_run("-v","-s")
+        confcut = "--confcutdir={0}".format(testdir.tmpdir)
+        reprec = testdir.inline_run("-v","-s", confcut)
         reprec.assertoutcome(passed=8)
         config = reprec.getcalls("pytest_unconfigure")[0].config
         l = config.pluginmanager._getconftestmodules(p)[0].l
@@ -1757,17 +1901,19 @@ class TestFixtureMarker:
     def test_scope_module_and_finalizer(self, testdir):
         testdir.makeconftest("""
             import pytest
-            finalized = []
-            created = []
+            finalized_list = []
+            created_list = []
             @pytest.fixture(scope="module")
             def arg(request):
-                created.append(1)
+                created_list.append(1)
                 assert request.scope == "module"
-                request.addfinalizer(lambda: finalized.append(1))
-            def pytest_funcarg__created(request):
-                return len(created)
-            def pytest_funcarg__finalized(request):
-                return len(finalized)
+                request.addfinalizer(lambda: finalized_list.append(1))
+            @pytest.fixture
+            def created(request):
+                return len(created_list)
+            @pytest.fixture
+            def finalized(request):
+                return len(finalized_list)
         """)
         testdir.makepyfile(
             test_mod1="""
@@ -1790,9 +1936,9 @@ class TestFixtureMarker:
         reprec.assertoutcome(passed=4)
 
     @pytest.mark.parametrize("method", [
-        'request.getfuncargvalue("arg")',
+        'request.getfixturevalue("arg")',
         'request.cached_setup(lambda: None, scope="function")',
-    ], ids=["getfuncargvalue", "cached_setup"])
+    ], ids=["getfixturevalue", "cached_setup"])
     def test_scope_mismatch_various(self, testdir, method):
         testdir.makeconftest("""
             import pytest
@@ -2083,7 +2229,7 @@ class TestFixtureMarker:
                 return {}
         """)
         b = testdir.mkdir("subdir")
-        b.join("test_overriden_fixture_finalizer.py").write(dedent("""
+        b.join("test_overridden_fixture_finalizer.py").write(dedent("""
             import pytest
             @pytest.fixture
             def browser(browser):
@@ -2597,11 +2743,13 @@ class TestShowFixtures:
         ''')
 
 
+@pytest.mark.parametrize('flavor', ['fixture', 'yield_fixture'])
 class TestContextManagerFixtureFuncs:
-    def test_simple(self, testdir):
+
+    def test_simple(self, testdir, flavor):
         testdir.makepyfile("""
             import pytest
-            @pytest.yield_fixture
+            @pytest.{flavor}
             def arg1():
                 print ("setup")
                 yield 1
@@ -2611,7 +2759,7 @@ class TestContextManagerFixtureFuncs:
             def test_2(arg1):
                 print ("test2 %s" % arg1)
                 assert 0
-        """)
+        """.format(flavor=flavor))
         result = testdir.runpytest("-s")
         result.stdout.fnmatch_lines("""
             *setup*
@@ -2622,10 +2770,10 @@ class TestContextManagerFixtureFuncs:
             *teardown*
         """)
 
-    def test_scoped(self, testdir):
+    def test_scoped(self, testdir, flavor):
         testdir.makepyfile("""
             import pytest
-            @pytest.yield_fixture(scope="module")
+            @pytest.{flavor}(scope="module")
             def arg1():
                 print ("setup")
                 yield 1
@@ -2634,7 +2782,7 @@ class TestContextManagerFixtureFuncs:
                 print ("test1 %s" % arg1)
             def test_2(arg1):
                 print ("test2 %s" % arg1)
-        """)
+        """.format(flavor=flavor))
         result = testdir.runpytest("-s")
         result.stdout.fnmatch_lines("""
             *setup*
@@ -2643,83 +2791,171 @@ class TestContextManagerFixtureFuncs:
             *teardown*
         """)
 
-    def test_setup_exception(self, testdir):
+    def test_setup_exception(self, testdir, flavor):
         testdir.makepyfile("""
             import pytest
-            @pytest.yield_fixture(scope="module")
+            @pytest.{flavor}(scope="module")
             def arg1():
                 pytest.fail("setup")
                 yield 1
             def test_1(arg1):
                 pass
-        """)
+        """.format(flavor=flavor))
         result = testdir.runpytest("-s")
         result.stdout.fnmatch_lines("""
             *pytest.fail*setup*
             *1 error*
         """)
 
-    def test_teardown_exception(self, testdir):
+    def test_teardown_exception(self, testdir, flavor):
         testdir.makepyfile("""
             import pytest
-            @pytest.yield_fixture(scope="module")
+            @pytest.{flavor}(scope="module")
             def arg1():
                 yield 1
                 pytest.fail("teardown")
             def test_1(arg1):
                 pass
-        """)
+        """.format(flavor=flavor))
         result = testdir.runpytest("-s")
         result.stdout.fnmatch_lines("""
             *pytest.fail*teardown*
             *1 passed*1 error*
         """)
 
-    def test_yields_more_than_one(self, testdir):
+    def test_yields_more_than_one(self, testdir, flavor):
         testdir.makepyfile("""
             import pytest
-            @pytest.yield_fixture(scope="module")
+            @pytest.{flavor}(scope="module")
             def arg1():
                 yield 1
                 yield 2
             def test_1(arg1):
                 pass
-        """)
+        """.format(flavor=flavor))
         result = testdir.runpytest("-s")
         result.stdout.fnmatch_lines("""
             *fixture function*
             *test_yields*:2*
         """)
 
-
-    def test_no_yield(self, testdir):
+    def test_custom_name(self, testdir, flavor):
         testdir.makepyfile("""
             import pytest
-            @pytest.yield_fixture(scope="module")
+            @pytest.{flavor}(name='meow')
             def arg1():
-                return 1
-            def test_1(arg1):
-                pass
-        """)
+                return 'mew'
+            def test_1(meow):
+                print(meow)
+        """.format(flavor=flavor))
         result = testdir.runpytest("-s")
-        result.stdout.fnmatch_lines("""
-            *yield_fixture*requires*yield*
-            *yield_fixture*
-            *def arg1*
-        """)
+        result.stdout.fnmatch_lines("*mew*")
 
-    def test_yield_not_allowed_in_non_yield(self, testdir):
-        testdir.makepyfile("""
+class TestParameterizedSubRequest:
+    def test_call_from_fixture(self, testdir):
+        testfile = testdir.makepyfile("""
             import pytest
-            @pytest.fixture(scope="module")
-            def arg1():
-                yield 1
-            def test_1(arg1):
+
+            @pytest.fixture(params=[0, 1, 2])
+            def fix_with_param(request):
+                return request.param
+
+            @pytest.fixture
+            def get_named_fixture(request):
+                return request.getfixturevalue('fix_with_param')
+
+            def test_foo(request, get_named_fixture):
                 pass
-        """)
-        result = testdir.runpytest("-s")
+            """)
+        result = testdir.runpytest()
         result.stdout.fnmatch_lines("""
-            *fixture*cannot use*yield*
-            *def arg1*
-        """)
+            E*Failed: The requested fixture has no parameter defined for the current test.
+            E*
+            E*Requested fixture 'fix_with_param' defined in:
+            E*{0}:4
+            E*Requested here:
+            E*{1}:9
+            *1 error*
+            """.format(testfile.basename, testfile.basename))
+
+    def test_call_from_test(self, testdir):
+        testfile = testdir.makepyfile("""
+            import pytest
+
+            @pytest.fixture(params=[0, 1, 2])
+            def fix_with_param(request):
+                return request.param
+
+            def test_foo(request):
+                request.getfixturevalue('fix_with_param')
+            """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines("""
+            E*Failed: The requested fixture has no parameter defined for the current test.
+            E*
+            E*Requested fixture 'fix_with_param' defined in:
+            E*{0}:4
+            E*Requested here:
+            E*{1}:8
+            *1 failed*
+            """.format(testfile.basename, testfile.basename))
+
+    def test_external_fixture(self, testdir):
+        conffile = testdir.makeconftest("""
+            import pytest
+
+            @pytest.fixture(params=[0, 1, 2])
+            def fix_with_param(request):
+                return request.param
+            """)
+
+        testfile = testdir.makepyfile("""
+            def test_foo(request):
+                request.getfixturevalue('fix_with_param')
+            """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines("""
+            E*Failed: The requested fixture has no parameter defined for the current test.
+            E*
+            E*Requested fixture 'fix_with_param' defined in:
+            E*{0}:4
+            E*Requested here:
+            E*{1}:2
+            *1 failed*
+            """.format(conffile.basename, testfile.basename))
+
+    def test_non_relative_path(self, testdir):
+        tests_dir = testdir.mkdir('tests')
+        fixdir = testdir.mkdir('fixtures')
+        fixfile = fixdir.join("fix.py")
+        fixfile.write(_pytest._code.Source("""
+            import pytest
+
+            @pytest.fixture(params=[0, 1, 2])
+            def fix_with_param(request):
+                return request.param
+            """))
+
+        testfile = tests_dir.join("test_foos.py")
+        testfile.write(_pytest._code.Source("""
+            from fix import fix_with_param
+
+            def test_foo(request):
+                request.getfixturevalue('fix_with_param')
+            """))
+
+        tests_dir.chdir()
+        testdir.syspathinsert(fixdir)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines("""
+            E*Failed: The requested fixture has no parameter defined for the current test.
+            E*
+            E*Requested fixture 'fix_with_param' defined in:
+            E*{0}:5
+            E*Requested here:
+            E*{1}:5
+            *1 failed*
+            """.format(fixfile.strpath, testfile.basename))
+
+
 
