@@ -2,12 +2,11 @@
 import sys
 import textwrap
 
-import py, pytest
 import _pytest.assertion as plugin
-from _pytest.assertion import reinterpret
+import py
+import pytest
 from _pytest.assertion import util
 
-needsnewassert = pytest.mark.skipif("sys.version_info < (2,6)")
 PY3 = sys.version_info >= (3, 0)
 
 
@@ -22,25 +21,229 @@ def mock_config():
     return Config()
 
 
-def interpret(expr):
-    return reinterpret.reinterpret(expr, py.code.Frame(sys._getframe(1)))
+class TestImportHookInstallation:
+
+    @pytest.mark.parametrize('initial_conftest', [True, False])
+    @pytest.mark.parametrize('mode', ['plain', 'rewrite'])
+    def test_conftest_assertion_rewrite(self, testdir, initial_conftest, mode):
+        """Test that conftest files are using assertion rewrite on import.
+        (#1619)
+        """
+        testdir.tmpdir.join('foo/tests').ensure(dir=1)
+        conftest_path = 'conftest.py' if initial_conftest else 'foo/conftest.py'
+        contents = {
+            conftest_path: """
+                import pytest
+                @pytest.fixture
+                def check_first():
+                    def check(values, value):
+                        assert values.pop(0) == value
+                    return check
+            """,
+            'foo/tests/test_foo.py': """
+                def test(check_first):
+                    check_first([10, 30], 30)
+            """
+        }
+        testdir.makepyfile(**contents)
+        result = testdir.runpytest_subprocess('--assert=%s' % mode)
+        if mode == 'plain':
+            expected = 'E       AssertionError'
+        elif mode == 'rewrite':
+            expected = '*assert 10 == 30*'
+        else:
+            assert 0
+        result.stdout.fnmatch_lines([expected])
+
+    @pytest.mark.parametrize('mode', ['plain', 'rewrite'])
+    def test_pytest_plugins_rewrite(self, testdir, mode):
+        contents = {
+            'conftest.py': """
+                pytest_plugins = ['ham']
+            """,
+            'ham.py': """
+                import pytest
+                @pytest.fixture
+                def check_first():
+                    def check(values, value):
+                        assert values.pop(0) == value
+                    return check
+            """,
+            'test_foo.py': """
+                def test_foo(check_first):
+                    check_first([10, 30], 30)
+            """,
+        }
+        testdir.makepyfile(**contents)
+        result = testdir.runpytest_subprocess('--assert=%s' % mode)
+        if mode == 'plain':
+            expected = 'E       AssertionError'
+        elif mode == 'rewrite':
+            expected = '*assert 10 == 30*'
+        else:
+            assert 0
+        result.stdout.fnmatch_lines([expected])
+
+    @pytest.mark.parametrize('mode', ['str', 'list'])
+    def test_pytest_plugins_rewrite_module_names(self, testdir, mode):
+        """Test that pluginmanager correct marks pytest_plugins variables
+        for assertion rewriting if they are defined as plain strings or
+        list of strings (#1888).
+        """
+        plugins = '"ham"' if mode == 'str' else '["ham"]'
+        contents = {
+            'conftest.py': """
+                pytest_plugins = {plugins}
+            """.format(plugins=plugins),
+            'ham.py': """
+                import pytest
+            """,
+            'test_foo.py': """
+                def test_foo(pytestconfig):
+                    assert 'ham' in pytestconfig.pluginmanager.rewrite_hook._must_rewrite
+            """,
+        }
+        testdir.makepyfile(**contents)
+        result = testdir.runpytest_subprocess('--assert=rewrite')
+        assert result.ret == 0
+
+    @pytest.mark.parametrize('mode', ['plain', 'rewrite'])
+    def test_installed_plugin_rewrite(self, testdir, mode):
+        # Make sure the hook is installed early enough so that plugins
+        # installed via setuptools are re-written.
+        testdir.tmpdir.join('hampkg').ensure(dir=1)
+        contents = {
+            'hampkg/__init__.py': """
+                import pytest
+
+                @pytest.fixture
+                def check_first2():
+                    def check(values, value):
+                        assert values.pop(0) == value
+                    return check
+            """,
+            'spamplugin.py': """
+            import pytest
+            from hampkg import check_first2
+
+            @pytest.fixture
+            def check_first():
+                def check(values, value):
+                    assert values.pop(0) == value
+                return check
+            """,
+            'mainwrapper.py': """
+            import pytest, pkg_resources
+
+            class DummyDistInfo:
+                project_name = 'spam'
+                version = '1.0'
+
+                def _get_metadata(self, name):
+                    return ['spamplugin.py,sha256=abc,123',
+                            'hampkg/__init__.py,sha256=abc,123']
+
+            class DummyEntryPoint:
+                name = 'spam'
+                module_name = 'spam.py'
+                attrs = ()
+                extras = None
+                dist = DummyDistInfo()
+
+                def load(self, require=True, *args, **kwargs):
+                    import spamplugin
+                    return spamplugin
+
+            def iter_entry_points(name):
+                yield DummyEntryPoint()
+
+            pkg_resources.iter_entry_points = iter_entry_points
+            pytest.main()
+            """,
+            'test_foo.py': """
+            def test(check_first):
+                check_first([10, 30], 30)
+
+            def test2(check_first2):
+                check_first([10, 30], 30)
+            """,
+        }
+        testdir.makepyfile(**contents)
+        result = testdir.run(sys.executable, 'mainwrapper.py', '-s', '--assert=%s' % mode)
+        if mode == 'plain':
+            expected = 'E       AssertionError'
+        elif mode == 'rewrite':
+            expected = '*assert 10 == 30*'
+        else:
+            assert 0
+        result.stdout.fnmatch_lines([expected])
+
+    def test_rewrite_ast(self, testdir):
+        testdir.tmpdir.join('pkg').ensure(dir=1)
+        contents = {
+            'pkg/__init__.py': """
+                import pytest
+                pytest.register_assert_rewrite('pkg.helper')
+            """,
+            'pkg/helper.py': """
+                def tool():
+                    a, b = 2, 3
+                    assert a == b
+            """,
+            'pkg/plugin.py': """
+                import pytest, pkg.helper
+                @pytest.fixture
+                def tool():
+                    return pkg.helper.tool
+            """,
+            'pkg/other.py': """
+                l = [3, 2]
+                def tool():
+                    assert l.pop() == 3
+            """,
+            'conftest.py': """
+                pytest_plugins = ['pkg.plugin']
+            """,
+            'test_pkg.py': """
+                import pkg.other
+                def test_tool(tool):
+                    tool()
+                def test_other():
+                    pkg.other.tool()
+            """,
+        }
+        testdir.makepyfile(**contents)
+        result = testdir.runpytest_subprocess('--assert=rewrite')
+        result.stdout.fnmatch_lines(['>*assert a == b*',
+                                     'E*assert 2 == 3*',
+                                     '>*assert l.pop() == 3*',
+                                     'E*AssertionError'])
+
+    def test_register_assert_rewrite_checks_types(self):
+        with pytest.raises(TypeError):
+            pytest.register_assert_rewrite(['pytest_tests_internal_non_existing'])
+        pytest.register_assert_rewrite('pytest_tests_internal_non_existing',
+                                       'pytest_tests_internal_non_existing2')
+
 
 class TestBinReprIntegration:
-    pytestmark = needsnewassert
 
     def test_pytest_assertrepr_compare_called(self, testdir):
         testdir.makeconftest("""
+            import pytest
             l = []
             def pytest_assertrepr_compare(op, left, right):
                 l.append((op, left, right))
-            def pytest_funcarg__l(request):
+
+            @pytest.fixture
+            def list(request):
                 return l
         """)
         testdir.makepyfile("""
             def test_hello():
                 assert 0 == 1
-            def test_check(l):
-                assert l == [("==", 0, 1)]
+            def test_check(list):
+                assert list == [("==", 0, 1)]
         """)
         result = testdir.runpytest("-v")
         result.stdout.fnmatch_lines([
@@ -242,6 +445,9 @@ class TestAssert_reprcompare:
         expl = callequal(A(), '1')
         assert expl
 
+    def test_format_nonascii_explanation(self):
+        assert util.format_explanation('λ')
+
     def test_mojibake(self):
         # issue 429
         left = 'e'
@@ -361,7 +567,6 @@ def test_python25_compile_issue257(testdir):
             *1 failed*
     """)
 
-@needsnewassert
 def test_rewritten(testdir):
     testdir.makepyfile("""
         def test_rewritten():
@@ -374,7 +579,6 @@ def test_reprcompare_notin(mock_config):
         mock_config, 'not in', 'foo', 'aaafoobbb')[1:]
     assert detail == ["'foo' is contained here:", '  aaafoobbb', '?    +++']
 
-@needsnewassert
 def test_pytest_assertrepr_compare_integration(testdir):
     testdir.makepyfile("""
         def test_hello():
@@ -391,7 +595,6 @@ def test_pytest_assertrepr_compare_integration(testdir):
         "*E*50*",
     ])
 
-@needsnewassert
 def test_sequence_comparison_uses_repr(testdir):
     testdir.makepyfile("""
         def test_hello():
@@ -410,8 +613,7 @@ def test_sequence_comparison_uses_repr(testdir):
     ])
 
 
-@pytest.mark.xfail("sys.version_info < (2,6)")
-def test_assert_compare_truncate_longmessage(testdir):
+def test_assert_compare_truncate_longmessage(monkeypatch, testdir):
     testdir.makepyfile(r"""
         def test_long():
             a = list(range(200))
@@ -420,6 +622,7 @@ def test_assert_compare_truncate_longmessage(testdir):
             b = '\n'.join(map(str, b))
             assert a == b
     """)
+    monkeypatch.delenv('CI', raising=False)
 
     result = testdir.runpytest()
     # without -vv, truncate the message showing a few diff lines only
@@ -428,7 +631,7 @@ def test_assert_compare_truncate_longmessage(testdir):
         "*- 3",
         "*- 5",
         "*- 7",
-        "*truncated (191 more lines)*use*-vv*",
+        "*truncated (193 more lines)*use*-vv*",
     ])
 
 
@@ -437,8 +640,13 @@ def test_assert_compare_truncate_longmessage(testdir):
         "*- 197",
     ])
 
+    monkeypatch.setenv('CI', '1')
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines([
+        "*- 197",
+    ])
+
 
-@needsnewassert
 def test_assertrepr_loaded_per_dir(testdir):
     testdir.makepyfile(test_base=['def test_base(): assert 1 == 2'])
     a = testdir.mkdir('a')
@@ -469,24 +677,8 @@ def test_assertion_options(testdir):
     """)
     result = testdir.runpytest()
     assert "3 == 4" in result.stdout.str()
-    off_options = (("--no-assert",),
-                   ("--nomagic",),
-                   ("--no-assert", "--nomagic"),
-                   ("--assert=plain",),
-                   ("--assert=plain", "--no-assert"),
-                   ("--assert=plain", "--nomagic"),
-                   ("--assert=plain", "--no-assert", "--nomagic"))
-    for opt in off_options:
-        result = testdir.runpytest_subprocess(*opt)
-        assert "3 == 4" not in result.stdout.str()
-
-def test_old_assert_mode(testdir):
-    testdir.makepyfile("""
-        def test_in_old_mode():
-            assert "@py_builtins" not in globals()
-    """)
-    result = testdir.runpytest_subprocess("--assert=reinterp")
-    assert result.ret == 0
+    result = testdir.runpytest_subprocess("--assert=plain")
+    assert "3 == 4" not in result.stdout.str()
 
 def test_triple_quoted_string_issue113(testdir):
     testdir.makepyfile("""
@@ -547,14 +739,14 @@ def test_traceback_failure(testdir):
         "*test_traceback_failure.py:4: AssertionError"
     ])
 
-@pytest.mark.skipif("sys.version_info < (2,5) or '__pypy__' in sys.builtin_module_names or sys.platform.startswith('java')" )
+@pytest.mark.skipif("'__pypy__' in sys.builtin_module_names or sys.platform.startswith('java')" )
 def test_warn_missing(testdir):
     testdir.makepyfile("")
     result = testdir.run(sys.executable, "-OO", "-m", "pytest", "-h")
     result.stderr.fnmatch_lines([
         "*WARNING*assert statements are not executed*",
     ])
-    result = testdir.run(sys.executable, "-OO", "-m", "pytest", "--no-assert")
+    result = testdir.run(sys.executable, "-OO", "-m", "pytest")
     result.stderr.fnmatch_lines([
         "*WARNING*assert statements are not executed*",
     ])
@@ -621,3 +813,44 @@ def test_set_with_unsortable_elements():
         + repr(3)
     """).strip()
     assert '\n'.join(expl) == dedent
+
+def test_diff_newline_at_end(monkeypatch, testdir):
+    testdir.makepyfile(r"""
+        def test_diff():
+            assert 'asdf' == 'asdf\n'
+    """)
+
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines(r"""
+        *assert 'asdf' == 'asdf\n'
+        *  - asdf
+        *  + asdf
+        *  ?     +
+    """)
+
+def test_assert_tuple_warning(testdir):
+    testdir.makepyfile("""
+        def test_tuple():
+            assert(False, 'you shall not pass')
+    """)
+    result = testdir.runpytest('-rw')
+    result.stdout.fnmatch_lines('WR1*:2 assertion is always true*')
+
+def test_assert_indirect_tuple_no_warning(testdir):
+    testdir.makepyfile("""
+        def test_tuple():
+            tpl = ('foo', 'bar')
+            assert tpl
+    """)
+    result = testdir.runpytest('-rw')
+    output = '\n'.join(result.stdout.lines)
+    assert 'WR1' not in output
+
+def test_assert_with_unicode(monkeypatch, testdir):
+    testdir.makepyfile(u"""
+        # -*- coding: utf-8 -*-
+        def test_unicode():
+            assert u'유니코드' == u'Unicode'
+    """)
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines(['*AssertionError*'])
